/*
 * --- file pushButton.h ---
 *
 * This is the file header for the push button connected to the LPC1769
 * @Author: Carles Saura
 */

#ifndef __PUSHBUTTON_H
#define __PUSHBUTTON_H

/*-----------------------------------------------------------*/
/*
 * Simple function to init the pin of the port given by parameters to activate the interruptions.
 * Then you must to write a part of the correlative code in EINT3_IRQHandler(), ie.:
 * void EINT3_IRQHandler (void)
 *{
 *	if ((LPC_GPIOINT->IO0IntStatR & (1 << 6)) == (1 << 6)) // the interruption was raised by P0.6
 *	{
 *		LPC_GPIOINT->IO0IntClr |= (1 << 6);  // clear the status
 *		... // do your task
 *	}
 *}
 * @precondition port must value 0 or 2, otherwise will be set as default (0).
 *
 * @param port  		Port to initialize the connection
 * @param pin			The pin where the push button is connected
 */
void pushButton_startInt( uint8_t  port, uint8_t pin );

/*
 * Simple function to check if interrupt is Raised with and implemented method to debounce

 * @precondition port must value 0 or 2, otherwise will be set as default (0).
 *
 * @param port  		Port to initialize the connection
 * @param pin			The pin where the push button is connected
 *
 * @return 				1 if is stable, otherwise 0
 */
uint8_t pushButton_raiseInt( uint8_t  port, uint8_t pin );

/*
 * Simple function to check if interrupt is Failling with and implemented method to debounce

 * @precondition port must value 0 or 2, otherwise will be set as default (0).
 *
 * @param port  		Port to initialize the connection
 * @param pin			The pin where the push button is connected
 *
 * @return 				1 if is stable, otherwise 0
 */
void pushButton_fallingInt( uint8_t  port, uint8_t pin );

uint8_t pushButton_timerCallback( unsigned long *toSend,  unsigned long *toSet );
/*-----------------------------------------------------------*/
#endif
