/*
 * This is the file implementation for interface with the RN-XV module using UART
 * @Author: Carles Saura
 */
#include "rnxv.h"
/* CLib includes */
#include <stdarg.h>
#include "FreeRTOS.h"
#include "task.h"
/* to print using UART include */
#include "printf.h"
uint8_t uart_port = 3;
portTickType pxPreviousWakeTime = -1;
/* The rate at which data is sent to the queue, specified in milliseconds. */
#define logTIME_TO_WAIT			( 100 / portTICK_RATE_MS )

/*-----------------------------------------------------------*/

void rnxv_join( uint8_t port, char*  ssid ){
	 Printf_start( port, 115200);
	 uart_port = port;
	 /* Initialise pxPreviousWakeTime - this only needs to be done once. */
	 //if ( pxPreviousWakeTime != -1 )
		 pxPreviousWakeTime = xTaskGetTickCount();
}


void rnxv_log(char* str, const char* format, ...){
	const char *logHead = "\n***********************************\nText Log:\n\b";
	const char *logEnd = "\n*******************************end*\n\n";
	uint8_t trans_id;
	va_list listPointer;
	/* This is used to transmit the variable parameters */
	va_start( listPointer, format );
	/* Enable transaction mode */
	trans_id = Printf_transactionOn();
	while ( trans_id == 0 ){
		/* Wait */
		vTaskDelayUntil( &pxPreviousWakeTime, logTIME_TO_WAIT );
		trans_id = Printf_transactionOn();
	}
	Printf_print( uart_port, trans_id, str, logHead );
	Printf_vsprint( uart_port, trans_id, str,format, listPointer );
	Printf_print( uart_port, trans_id, str, logEnd );
	Printf_transactionOff();
}

/*-----------------------------------------------------------*/
