/*
 * --- file timer.c ---
 *
 * This is the file header for the timer module. This module is a layer to setup the four
 * timers that the LPC1769 has (TIMER0/1/2/3). This code is based on a Code Red Technologies
 * example, but all the prototypes and functions are being rewrited
 * @Author: Carles Saura
 */

#include "LPC17xx.h"
#include "timer.h"
#ifndef __CUSTOM_UTILS_H
	#include "utils.h"
#endif

volatile uint32_t timer0_counter = 0;
volatile uint32_t timer1_counter = 0;
volatile uint32_t timer2_counter = 0;
volatile uint32_t timer3_counter = 0;
/*-----------------------------------------------------------*/

void timer_power( uint8_t timer_num )
{
	switch ( timer_num ){
		case 0: LPC_SC->PCONP |= 1 << 1; break;
		case 1: LPC_SC->PCONP |= 1 << 2; break;
		case 2: LPC_SC->PCONP |= 1 << 22; break;
		default:
		case 3: LPC_SC->PCONP |= 1 << 23; break;
	}
}

void timer_reset( uint8_t timer_num )
{
	switch ( timer_num ){
		case 0: LPC_TIM0->TCR |= _SBF( 1, 1 ); break;
		case 1: LPC_TIM1->TCR |= _SBF( 1, 1 ); break;
		case 2: LPC_TIM2->TCR |= _SBF( 1, 1 ); break;
		default:
		case 3: LPC_TIM3->TCR |= _SBF( 1, 1 );; break;
	}

}

void timer_prescaler(uint8_t timer_num, uint8_t timer_setting)
{
    switch ( timer_num ){
    case 0:
    	/* Clear two bit at bit position 2 */
    	LPC_SC->PCLKSEL0 &= ~(_SBF( 2, 0x03 ));
    	/* Set two selected bit */
    	LPC_SC->PCLKSEL0 |= _SBF( 2,timer_setting );
    	break;
    case 1:
    	/* Clear two bit at bit position 4 */
    	LPC_SC->PCLKSEL0 &= ~(_SBF( 4, 0x03 ));
    	/* Set two selected bit */
    	LPC_SC->PCLKSEL0 |= _SBF( 4, timer_setting );
    	break;
    case 2:
    	/* Clear two bit at bit position 12 */
    	LPC_SC->PCLKSEL1 &= ~(_SBF( 12, 0x03 ));
    	/* Set two selected bit */
    	LPC_SC->PCLKSEL1 |= _SBF( 12, timer_setting );
    	break;
    case 3:
    	/* Clear two bit at bit position 14 */
    	LPC_SC->PCLKSEL1 &= ~(_SBF( 14, 0x03 ));
    	/* Set two selected bit */
    	LPC_SC->PCLKSEL1 |= _SBF( 14, timer_setting );
    	break;
    }
}

void timer_init ( uint8_t timer_num, uint32_t TimerInterval )
{
	switch ( timer_num ){
		case 0:
			timer0_counter = 0;
			LPC_TIM0->MR0 = TimerInterval;
			LPC_TIM0->MCR = _SBF( 0, 0x03 ); // Interrupt and Reset on MR0
			LPC_TIM0->TCR |= _SBF( 1, 1 );  // Reset the timer
			NVIC_EnableIRQ(TIMER0_IRQn);
			break;
		case 1:
			timer1_counter = 0;
			LPC_TIM1->MR0 = TimerInterval;
			LPC_TIM1->MCR = _SBF( 0, 0x03 );  // Interrupt and Reset on MR0
			LPC_TIM1->TCR |= _SBF( 1, 1 );  // Reset the timer
			NVIC_EnableIRQ(TIMER1_IRQn);
			break;
		case 2:
			timer2_counter = 0;
			LPC_TIM2->MR0 = TimerInterval;
			LPC_TIM2->MCR = _SBF( 0, 0x03 );  // Interrupt and Reset on MR0
			LPC_TIM3->TCR |= _SBF( 1, 1 );  // Reset the timer
			NVIC_EnableIRQ(TIMER2_IRQn);
			break;
		default:
		case 3:
			timer3_counter = 0;
			LPC_TIM3->MR0 = TimerInterval;
			LPC_TIM3->MCR = _SBF( 0, 0x03 );  // Interrupt and Reset on MR0
			LPC_TIM3->TCR |= _SBF( 1, 1 );  // Reset the timer
			NVIC_EnableIRQ(TIMER3_IRQn);
			break;
	}
}

void timer_enable( uint8_t timer_num )
{
	switch ( timer_num ){
		case 0: LPC_TIM0->TCR = _SBF( 0, 1 ); break;
		case 1: LPC_TIM1->TCR = _SBF( 0, 1 ); break;
		case 2: LPC_TIM2->TCR = _SBF( 0, 1 ); break;
		default:
		case 3: LPC_TIM3->TCR = _SBF( 0, 1 ); break;
	}
}

void timer_disable( uint8_t timer_num )
{
	switch ( timer_num ){
		case 0: LPC_TIM0->TCR = 0; break;
		case 1: LPC_TIM1->TCR = 0; break;
		case 2: LPC_TIM2->TCR = 0; break;
		default:
		case 3: LPC_TIM3->TCR = 0; break;
	}
}
