/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* FreeRTOS.org includes. */
#ifndef INC_FREERTOS_H
	#include "FreeRTOS.h"
#endif
#ifndef TASK_H
	#include "task.h"
#endif
#ifndef SEMAPHORE_H
	#include "semphr.h"
#endif
/* Includes for testing */
#ifndef __STRING_H_INCLUDED
	#include <string.h>
#endif
#ifndef LPCXPRESSO1769
	#include "LPC17xx.h"
#endif
#ifndef __UART_H
	#include "uart.h"
#endif


/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

/* The task functions. */
void test1( void *pvParameters );
void test2( void *pvParameters );
void test3( void *pvParameters );
void test4( void *pvParameters );

/* Variables used to test */
extern volatile uint32_t UART0Count, UART1Count, UART2Count, UART3Count;
extern volatile uint8_t UART0Buffer, UART1Buffer, UART2Buffer, UART3Buffer;
volatile uint32_t *main_lpc_uart_ier;
volatile uint32_t *main_uart_count;
volatile uint8_t *main_uart_buffer;
uint8_t mainPort = 3;
int mainBaurade = 115200;
int done = 0;
xSemaphoreHandle sem_test1 = NULL;
/*-----------------------------------------------------------*/

int main( void )
{
	/* Init the semi-hosting. This is to debug only */
	printf( "\n" );

	/* Setting global poniters */
	switch(mainPort){
	case 0: main_lpc_uart_ier = &LPC_UART0->IER;
		main_uart_count = &UART0Count;
		main_uart_buffer = &UART0Buffer;
		break;
	case 1: main_lpc_uart_ier = &LPC_UART1->IER;
		main_uart_count = &UART1Count;
		main_uart_buffer = &UART1Buffer;
		break;
	case 2: main_lpc_uart_ier = &LPC_UART2->IER;
		main_uart_count = &UART2Count;
		main_uart_buffer = &UART2Buffer;
		break;
	default:
	case 3: main_lpc_uart_ier = &LPC_UART3->IER;
		main_uart_count = &UART3Count;
		main_uart_buffer = &UART3Buffer;
		break;
	}
	/* Initiate the test 1 semaphore */
	if ( sem_test1 == NULL ){
		vSemaphoreCreateBinary( sem_test1 );
		xSemaphoreTake( sem_test1, ( portTickType ) 10 );
	}


	/* Create the other task in exactly the same way as last. */
	xTaskCreate( test1, "Test 1", 240, NULL, 1, NULL );

	/* Create the other task in exactly the same way as last. */
	xTaskCreate( test2, "Test 2", 240, NULL, 1, NULL );

	/* Create the other task in exactly the same way as last. */
	xTaskCreate( test3, "Test 3", 240, NULL, 1, NULL );

	/* Create the other task in exactly the same way as last. */
	xTaskCreate( test4, "Test 4", 240, NULL, 1, NULL );


	/* Start the scheduler so our tasks start executing. */
	vTaskStartScheduler();

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

void test1( void *pvParameters )
{
const char *text = "Test 1 (UART)\n\n";
volatile unsigned long ul;

	/* As per most tasks, this task is implemented in an infinite loop. */
	int myiter;

	for( myiter = 0;;myiter++ )
	{
		if( done == 0 ) /* this test is only executed one time */
		{
			/* Starting */
			UARTInit(mainPort, mainBaurade);
			/* Print out the name of this test using UART. */
			*main_lpc_uart_ier = IER_THRE | IER_RLS;				/* Disable RBR */
			UARTSend(mainPort, (uint8_t *)text , strlen(text) );
			*main_uart_count = 0;
			*main_lpc_uart_ier = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			xSemaphoreGive( sem_test1 );
			done = 1; /* Prevent to execute again */
			/* Delay for a period. */
			for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
			{
				/* This loop is just a very crude delay implementation.  There is
				nothing to do in here.  Later exercises will replace this crude
				loop with a proper delay/sleep function. */
			}
		}
	}
}
/*-----------------------------------------------------------*/
void test2( void *pvParameters )
{
const char *pcTaskName = "Test 2 (Printf_vsprint) is running the iteration number: %d\n\n";
volatile unsigned long ul;

	/* As per most tasks, this task is implemented in an infinite loop. */
	int myiter;
	for( myiter = 0;;myiter++ )
	{
		if( done == 0 ){ /* If Test 1 is done, then... */
			while( xSemaphoreTake( sem_test1, ( portTickType ) 100 ) != pdTRUE ){}
			xSemaphoreGive( sem_test1 );
		}
		/* Initiate the UART port to print */
		Printf_start(mainPort, mainBaurade);
		/* Print out the name of this test using PRINTF. */
		char str[30];
		Printf_print( mainPort, 0, str, pcTaskName, myiter );

		/* Delay for a period. */
		for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
		{
			/* This loop is just a very crude delay implementation.  There is
			nothing to do in here.  Later exercises will replace this crude
			loop with a proper delay/sleep function. */
		}
	}
}
/*-----------------------------------------------------------*/

void test3( void *pvParameters )
{
const char *text1 = "\nTest 3 (printfScan) iteration: %d - Introduce some char...\n";
const char *text2 = " returned count: %d text is:\n";
char received[80] = "\0";
volatile unsigned long ul;
uint8_t trans_id;

	/* As per most tasks, this task is implemented in an infinite loop. */
	int myiter;
	int bufsz;
	for( myiter = 0;;myiter++ )
	{
		if( done == 0 ){ /* If Test 1 is not done, then wait */
			while( xSemaphoreTake( sem_test1, ( portTickType ) 100 ) != pdTRUE ){}
			xSemaphoreGive( sem_test1 );
		}
		/* Initiate the UART port to scan */
		Printf_start( mainPort, mainBaurade);
		/* Enable transaction mode */
		trans_id = Printf_transactionOn();

		char str[30];
		Printf_print( mainPort, trans_id, str, text1, myiter );
		/* Scan */
		bufsz = Printf_scanf(mainPort, received, 4000);
		received[bufsz] = '\n';  // End of line
		received[bufsz + 1] = '\n';
		received[bufsz + 2] = 0; // End of string
		/* Show results */
		Printf_print( mainPort, trans_id, str, text2, bufsz );
		Printf_print( mainPort, trans_id, str, received);
		/* Disable transaction mode */
		Printf_transactionOff();

		/* Delay for a period. */
		for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
		{
			/* This loop is just a very crude delay implementation.  There is
			nothing to do in here.  Later exercises will replace this crude
			loop with a proper delay/sleep function. */
		}
	}
}
/*-----------------------------------------------------------*/

void test4( void *pvParameters )
{
const char *pcTaskName = "Test 4 (Log) is running the iteration number: %d\n";
volatile unsigned long ul;

	/* As per most tasks, this task is implemented in an infinite loop. */
	int myiter;
	for( myiter = 0;;myiter++ )
	{
		if( done == 0 ){ /* If Test 1 is not done, then wait */
			while( xSemaphoreTake( sem_test1, ( portTickType ) 100 ) != pdTRUE ){}
			xSemaphoreGive( sem_test1 );
		}
		/* Initiate the UART port to print */
		Log_start(mainPort, mainBaurade);

		/* Print out the name of this test using LOG. */
		char str[30];
		Log_log( str, pcTaskName, myiter );

		/* Delay for a period. */
		for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
		{
			/* This loop is just a very crude delay implementation.  There is
			nothing to do in here.  Later exercises will replace this crude
			loop with a proper delay/sleep function. */
		}
	}
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


